=== modified file 'NEWS'
--- NEWS	2010-02-19 04:54:24 +0000
+++ NEWS	2010-02-25 04:51:05 +0000
@@ -352,18 +352,24 @@
   (Martin Pool)
 
 
-bzr 2.0.5 (not released yet)
-############################
+bzr 2.0.5
+#########
 
 :Codename:
-:2.0.5:
+:2.0.5: NOT RELEASED YET
 
 Bug Fixes
 *********
 
+* Avoid ``malloc(0)`` in ``patiencediff``, which is non-portable.
+  (Martin Pool, #331095)
+
 * Handle renames correctly when there are files or directories that 
   differ only in case.  (Chris Jones, Martin Pool, #368931)
 
+* Fixed CHM generation by moving the NEWS section template into
+  a separate file. (Ian Clatworthy, #524184)
+
 * If ``bzr push --create-prefix`` triggers an unexpected ``NoSuchFile``
   error, report that error rather than failing with an unhelpful
   ``UnboundLocalError``.
@@ -375,6 +381,22 @@
 * Added ``location-alias`` help topic.
   (Andrew Bennetts, #337834)
 
+API Changes
+***********
+
+* Added ``bzrlib.osutils.set_signal_handler``, a convenience function that
+  can set a signal handler and call ``signal.siginterrupt(signum,
+  False)`` for it, if the platform and Python version supports it.
+  (Andrew Bennetts, #496813)
+
+Internals
+*********
+
+* ``bzrlib.breakin`` uses ``bzrlib.osutils.set_signal_handler`` to set the
+  ``SIGQUIT`` signal handler, so that entering the debugger with
+  ``SIGQUIT`` will no longer cause ``EINTR`` during blocking IO.
+  (Andrew Bennetts, #173007)
+
 bzr 2.0.4
 #########
 
@@ -11853,37 +11875,5 @@
   diff, status, etc.
 
 
-bzr ?.?.? (not released yet)
-############################
-
-:Codename: template
-:2.0.2: ???
-
-Compatibility Breaks
-********************
-
-New Features
-************
-
-Bug Fixes
-*********
-
-Improvements
-************
-
-Documentation
-*************
-
-API Changes
-***********
-
-Internals
-*********
-
-Testing
-*******
-
-
-
 ..
    vim: tw=74 ft=rst ff=unix

=== added file 'NEWS-template.txt'
--- NEWS-template.txt	1970-01-01 00:00:00 +0000
+++ NEWS-template.txt	2010-02-19 07:13:45 +0000
@@ -0,0 +1,31 @@
+bzr ?.?.?
+#########
+
+:Codename: Nirvana
+:2.x.y: NOT RELEASED YET
+
+Compatibility Breaks
+********************
+
+New Features
+************
+
+Bug Fixes
+*********
+
+Improvements
+************
+
+Documentation
+*************
+
+API Changes
+***********
+
+Internals
+*********
+
+Testing
+*******
+
+

=== modified file 'bzrlib/_patiencediff_c.c'
--- bzrlib/_patiencediff_c.c	2009-10-29 06:11:23 +0000
+++ bzrlib/_patiencediff_c.c	2010-02-25 04:51:05 +0000
@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2007 Canonical Ltd
+ Copyright (C) 2007, 2010 Canonical Ltd
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -46,6 +46,12 @@
 #define SENTINEL -1
 
 
+/* malloc returns NULL on some platforms if you try to allocate nothing,
+ * causing <https://bugs.edge.launchpad.net/bzr/+bug/511267> and
+ * <https://bugs.edge.launchpad.net/bzr/+bug/331095>.  On glibc it passes, but
+ * let's make it fail to aid testing. */
+#define guarded_malloc(x) ( (x) ? malloc(x) : NULL )
+
 enum {
     OP_EQUAL = 0,
     OP_INSERT,
@@ -183,7 +189,8 @@
     while (hsize < bsize + 1)
         hsize *= 2;
 
-    hashtable = (struct bucket *)malloc(sizeof(struct bucket) * hsize);
+    /* can't be 0 */
+    hashtable = (struct bucket *) guarded_malloc(sizeof(struct bucket) * hsize);
     if (hashtable == NULL) {
         PyErr_NoMemory();
         return 0;
@@ -460,7 +467,7 @@
     last_a_pos = alo - 1;
     last_b_pos = blo - 1;
 
-    lcs = (struct matching_line *)malloc(sizeof(struct matching_line) * (bhi - blo));
+    lcs = (struct matching_line *)guarded_malloc(sizeof(struct matching_line) * (bhi - blo));
     if (lcs == NULL)
         return 0;
 
@@ -620,13 +627,15 @@
     if (!equate_lines(&hashtable, a, b, asize, bsize))
         goto error;
 
-    matches = (struct matching_line *)malloc(sizeof(struct matching_line) * bsize);
-    if (matches == NULL)
-        goto error;
+    if (bsize > 0) {
+        matches = (struct matching_line *)guarded_malloc(sizeof(struct matching_line) * bsize);
+        if (matches == NULL)
+            goto error;
 
-    backpointers = (Py_ssize_t *)malloc(sizeof(Py_ssize_t) * bsize * 4);
-    if (backpointers == NULL)
-        goto error;
+        backpointers = (Py_ssize_t *)guarded_malloc(sizeof(Py_ssize_t) * bsize * 4);
+        if (backpointers == NULL)
+            goto error;
+    }
 
     nmatches = unique_lcs(matches, &hashtable, backpointers, a, b, 0, 0, asize, bsize);
 
@@ -692,13 +701,19 @@
         goto error;
 
     matches.count = 0;
-    matches.matches = (struct matching_block *)malloc(sizeof(struct matching_block) * bsize);
-    if (matches.matches == NULL)
-        goto error;
-
-    backpointers = (Py_ssize_t *)malloc(sizeof(Py_ssize_t) * bsize * 4);
-    if (backpointers == NULL)
-        goto error;
+
+    if (bsize > 0) {
+        matches.matches = (struct matching_block *)guarded_malloc(sizeof(struct matching_block) * bsize);
+        if (matches.matches == NULL)
+            goto error;
+
+        backpointers = (Py_ssize_t *)guarded_malloc(sizeof(Py_ssize_t) * bsize * 4);
+        if (backpointers == NULL)
+            goto error;
+    } else {
+        matches.matches = NULL;
+        backpointers = NULL;
+    }
 
     res = recurse_matches(&matches, &hashtable, backpointers,
                           a, b, alo, blo, ahi, bhi, maxrecursion);
@@ -765,11 +780,15 @@
             return NULL;
         }
 
-        self->backpointers = (Py_ssize_t *)malloc(sizeof(Py_ssize_t) * self->bsize * 4);
-        if (self->backpointers == NULL) {
-            Py_DECREF(self);
-            PyErr_NoMemory();
-            return NULL;
+        if (self->bsize > 0) {
+            self->backpointers = (Py_ssize_t *)guarded_malloc(sizeof(Py_ssize_t) * self->bsize * 4);
+            if (self->backpointers == NULL) {
+                Py_DECREF(self);
+                PyErr_NoMemory();
+                return NULL;
+            }
+        } else {
+            self->backpointers = NULL;
         }
 
     }
@@ -812,9 +831,13 @@
     struct matching_blocks matches;
 
     matches.count = 0;
-    matches.matches = (struct matching_block *)malloc(sizeof(struct matching_block) * self->bsize);
-    if (matches.matches == NULL)
-        return PyErr_NoMemory();
+    if (self->bsize > 0) {
+        matches.matches = (struct matching_block *)
+            guarded_malloc(sizeof(struct matching_block) * self->bsize);
+        if (matches.matches == NULL)
+            return PyErr_NoMemory();
+    } else
+        matches.matches = NULL;
 
     res = recurse_matches(&matches, &self->hashtable, self->backpointers,
                           self->a, self->b, 0, 0,
@@ -900,7 +923,7 @@
     struct matching_blocks matches;
 
     matches.count = 0;
-    matches.matches = (struct matching_block *)malloc(sizeof(struct matching_block) * (self->bsize + 1));
+    matches.matches = (struct matching_block *)guarded_malloc(sizeof(struct matching_block) * (self->bsize + 1));
     if (matches.matches == NULL)
         return PyErr_NoMemory();
 
@@ -1013,7 +1036,7 @@
         return NULL;
 
     matches.count = 0;
-    matches.matches = (struct matching_block *)malloc(sizeof(struct matching_block) * (self->bsize + 1));
+    matches.matches = (struct matching_block *)guarded_malloc(sizeof(struct matching_block) * (self->bsize + 1));
     if (matches.matches == NULL)
         return PyErr_NoMemory();
 
@@ -1031,7 +1054,7 @@
     matches.count++;
 
     ncodes = 0;
-    codes = (struct opcode *)malloc(sizeof(struct opcode) * matches.count * 2);
+    codes = (struct opcode *)guarded_malloc(sizeof(struct opcode) * matches.count * 2);
     if (codes == NULL) {
         free(matches.matches);
         return PyErr_NoMemory();
@@ -1252,3 +1275,7 @@
     PyModule_AddObject(m, "PatienceSequenceMatcher_c",
                        (PyObject *)&PatienceSequenceMatcherType);
 }
+
+
+/* vim: sw=4 et 
+ */

=== modified file 'bzrlib/breakin.py'
--- bzrlib/breakin.py	2009-07-30 23:54:26 +0000
+++ bzrlib/breakin.py	2010-02-25 02:51:00 +0000
@@ -25,6 +25,7 @@
 def _debug(signal_number, interrupted_frame):
     import pdb
     import sys
+    from bzrlib.osutils import set_signal_handler
     sys.stderr.write("** %s received, entering debugger\n"
             "** Type 'c' to continue or 'q' to stop the process\n"
             "** Or %s again to quit (and possibly dump core)\n"
@@ -38,7 +39,7 @@
     try:
         pdb.set_trace()
     finally:
-        signal.signal(_breakin_signal_number, _debug)
+        set_signal_handler(_breakin_signal_number, _debug)
 
 
 def hook_sigquit():
@@ -90,4 +91,5 @@
     if sig is None:
         return
     # print 'hooking into %s' % (_breakin_signal_name,)
-    signal.signal(sig, _debug)
+    from bzrlib.osutils import set_signal_handler
+    set_signal_handler(sig, _debug)

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2010-02-17 17:11:16 +0000
+++ bzrlib/osutils.py	2010-02-25 04:51:05 +0000
@@ -1344,6 +1344,27 @@
     normalized_filename = _inaccessible_normalized_filename
 
 
+def set_signal_handler(signum, handler, restart_syscall=True):
+    """A wrapper for signal.signal that also calls siginterrupt(signum, False)
+    on platforms that support that.
+
+    :param restart_syscall: if set, allow syscalls interrupted by a signal to
+        automatically restart (by calling `signal.siginterrupt(signum,
+        False)`).  May be ignored if the feature is not available on this
+        platform or Python version.
+    """
+    old_handler = signal.signal(signum, handler)
+    if restart_syscall:
+        try:
+            siginterrupt = signal.siginterrupt
+        except AttributeError: # siginterrupt doesn't exist on this platform, or for this version of
+            # Python.
+            pass
+        else:
+            siginterrupt(signum, False)
+    return old_handler
+
+
 default_terminal_width = 80
 """The default terminal width for ttys.
 
@@ -1451,7 +1472,7 @@
             # the current design -- vila 20091216
             pass
         else:
-            signal.signal(signal.SIGWINCH, _terminal_size_changed)
+            set_signal_handler(signal.SIGWINCH, _terminal_size_changed)
         _registered_sigwinch = True
 
 

=== modified file 'doc/en/user-guide/introducing_bazaar.txt'
--- doc/en/user-guide/introducing_bazaar.txt	2010-01-04 02:25:11 +0000
+++ doc/en/user-guide/introducing_bazaar.txt	2010-02-25 04:51:05 +0000
@@ -134,7 +134,7 @@
 details on the commands and options available.
 
 .. _Bazaar in five minutes: ../mini-tutorial/index.html
-.. _Bazaar Quick Start Card: ../quick-reference/quick-start-summary.svg
+.. _Bazaar Quick Start Card: ../quick-reference/index.html
 .. _Bazaar User Reference: ../user-reference/index.html
 
 We hope you find this manual useful. If you have suggestions on how it

